/*************************************************************************************************************************
*									DS18B20.c
*
*
*
*
* WinAVR Vers.:			2007 05 25
*
* AVR-Studio Version:	4.13 SP1
*
* Autor: 				Erik M.
*
* Getestet:				ja, OHNE Multiplexer			
*
* Letzte nderung:		28.12.07
*
*
*
* Ein Danke an die mikrocontroller.net Community fr ihre Hilfe!!!!!!!!!!!!!!!!!!!!!!!
***************************************************************************************************************************/

#include "DS18B20.h"








/**************************************************************************************************************************
*
*Funktionsname:		reset_1wire
*
*Rckgabe:			Fehlercode (errorcode)
*
*Parameter:			keine
*
*Aufgabe:			Sendet einen Resetimpuls an Multiplexer A+B und generiert einen Fehlercode
*
*					Fehlercode:	0= alles ok
*								1= Fehler an Multiplexer A
*								2= Fehler an Multiplexer B
*								3= Fehler an Multiplexer A+B
*
*
* getestet: ja, funktioniert!
*
***************************************************************************************************************************/

uint8_t reset_1wire(void)
{
	uint8_t errorcode=0;
	
	//senden des Resetimpulses an die Multiplexer A+B (2 Sensoren)
	W1_DDR |= ((1<<W1_PIN_A) | (1<<W1_PIN_B));	// Datenrichtung auf Ausgang => 1-Wire "low"
	_delay_ms(0.500);								// 500 usec warten	min480/max960usec
	
	W1_DDR &= ~((1<<W1_PIN_A) | (1<<W1_PIN_B));	// Datenrichtung auf Eingang => 1-Wire "high"
	_delay_us(40);								// 65usec warten 	Sensor braucht	min15/max60usec
	_delay_us(25);		
	
	if((!(W1_IN & (1<<W1_PIN_A)) && (!(W1_IN & (1<<W1_PIN_B)))))	//wenn von beiden Multiplexer ein Presence Puls (entspricht bit_is_clear)
		errorcode=0;
		
	else if((W1_IN & (1<<W1_PIN_A)) && (W1_IN & (1<<W1_PIN_B)))	//wenn von beiden Multiplexer kein Presence Puls (entspricht bit_is_set)
		errorcode=3;
	
	else if(W1_IN & (1<<W1_PIN_A))					//wenn Presence Pulse von Multiplikator A nicht korrekt empfangen.
		errorcode=1;								//(Schreibweise entspricht bit_is_set)
			
	else											//ansonsten wurde Presence Pulse von Multiplikator B nicht korrekt empfangen
		errorcode=2;
													// 480 usec minimum
	_delay_ms(0.435);								//500-65usec = 435usec warten
	
	return errorcode;								// Fehlercode zurckgeben
}



/**************************************************************************************************************************
*
*Funktionsname:		init_1wire
*
*Rckgabe:			--
*
*Parameter:			--	
*
*Aufgabe:			Initialisiert die Adressleitungen der Multiplexer und 1wire Datenleitungen. 
*					Diese Fkt. ist nur einmal am Anfang des Hauptprogramms aufzurufen!
*					
*
*getestet:			ja, funktioniert
*
*
***************************************************************************************************************************/

inline void init_1wire(void)
{
	// Ausgnge auf "low" und Datenrichtung auf Ausgang => Intitialisierungskanal = Kanal 0
	
	ADR_0 &= ~(1<<ADR_0_PIN);					// Ausgnge auf 0
	ADR_1 &= ~(1<<ADR_1_PIN);
	ADR_2 &= ~(1<<ADR_2_PIN);
	
	
	ADR_0_DDR |= (1<<ADR_0_PIN);				// Datenrichtung aud Ausgang
	ADR_1_DDR |= (1<<ADR_1_PIN);
	ADR_2_DDR |= (1<<ADR_2_PIN);
	
	_delay_us(2);								// 2usec Umschaltzeit
	
	W1_PORT &= ~((1<<W1_PIN_A) | (1<<W1_PIN_B));// Ausgang auf low
	W1_DDR &= ~((1<<W1_PIN_A) | (1<<W1_PIN_B));	// Datenrichtung auf Eingang >> 1wire high
	
	
}




/**************************************************************************************************************************
*
*Funktionsname:		temp_channel
*
*Rckgabe:			--
*
*Parameter:			bergeben wird der Temparaturkanal (0 bis 7) af den geschaltet werden soll
*
*Aufgabe:			stellt den gewnschten Kanal an den Multiplexer ein.
*
*
* getestet: ja, funktioniert!
*
* Optimierter Code von Falk Brunner
***************************************************************************************************************************/



void temp_channel(uint8_t ch)
{
	if (ch & 0x01)
		ADR_0 |= (1<<ADR_0_PIN);
	else
		ADR_0 &= ~(1<<ADR_0_PIN);
		
	
	if (ch & 0x02)
		ADR_1 |= (1<<ADR_1_PIN);
	else
		ADR_1 &= ~(1<<ADR_1_PIN);
		
	if (ch & 0x04)
		ADR_2 |= (1<<ADR_2_PIN);
	else
		ADR_2 &= ~(1<<ADR_2_PIN);
		
	
	_delay_us(2);
}






/**************************************************************************************************************************
*
*Funktionsname:		reset_all_1wire
*
*Rckgabe:			--
*
*Parameter:			--
*
*Aufgabe:			resetet alle Sensoren (egal ob angeschossen oder nicht) und erzeugt eine Sammlung aller Fehlercodes,
*					welche mittels der Fkt. show_errorcode_1wire() ber die serielle Schnittstelle angezeigt werden kann. 
*
*
*getestet:			ja, funktioniert!
*
*
***************************************************************************************************************************/

void reset_all_1wire(void)
{
	//static uint8_t errorcode[8];				//Feld mit 8x1 Byte  =>funktioniert hier mit static nicht?
												// wurde global im Header-File definiert!
	uint8_t i,c;
	
	for(i=0;i<=7;i++)							//Indizes laufen von 0 bis 7!
		errorcode[i]=0;							//Alle Fleder (errorcodes) auf 0 setzen


	for(c=0;c<=7;c++)						//schalte alle Kanle der Reihe nach durch; beginnend mit 0
	{
		temp_channel(c);
		
												// fhre einen Reset durch, erzeuge Fehlercode und
		errorcode[c]=(reset_1wire());			// speichere den Fehlercode des jeweiligen Kanals im dafr vorgesehenen Feld
	}
}








/**************************************************************************************************************************
*
*Funktionsname:		write_bit_1wire
*
*Rckgabe:			--
*
*Parameter:			zu schreibendes Bit
*
*Aufgabe:			schreibt 1 bit zu 2 Sensoren (Multiplexer A + B)
*
*
* getestet: ja, funktioniert
***************************************************************************************************************************/

void write_bit_1wire(uint8_t bit)
{
	uint8_t sreg_temp=0;
	
	sreg_temp = SREG;		// SREG sichern
	cli();					// Interrupt aus
	
	W1_DDR |= ((1<<W1_PIN_A) | (1<<W1_PIN_B));		// Datenrichtung auf Ausgang => 1-Wire "low"
	_delay_us(2);									// 2usec low Impuls

	if(bit == 1)									// wenn zu schreibendes bit eine 1 ist
		W1_DDR &= ~((1<<W1_PIN_A) | (1<<W1_PIN_B));	// Datenrichtung auf Eingang => 1Wire "high"
	
	_delay_us(40);									// Slot minimum 60usec
	_delay_us(25);
	W1_DDR &= ~((1<<W1_PIN_A) | (1<<W1_PIN_B));		// Datenrichtung auf Eingang => 1Wire "high"
	_delay_us(2);									// 2usec recovery Time

	SREG = sreg_temp;		// SREG zurckschreiben
}
	




/**************************************************************************************************************************
*
*Funktionsname:			write_byte_1wire
*
*Rckgabe:				--
*
*Parameter:				zu schreibendes Byte
*
*Aufgabe:				schreibt 1 Byte zu 2 Sensoren (Multiplexer A + B)
*
*
* getestet: ja,funktioniert
*
***************************************************************************************************************************/

void write_byte_1wire(uint8_t byte)
{
	uint8_t i, sende_bit=0;
		
	for (i=0;i<=7;i++)								// 8x1 bit schreiben = 1Byte
	{
		sende_bit = (byte >> i);					// schiebe das Byte um i-Stellen nach rechts
		sende_bit &=  0x01;							// Byte maskieren => nur das 0. bit bleibt brig
		write_bit_1wire(sende_bit);					// bit senden
	}
}




/**************************************************************************************************************************
*
*Funktionsname:			read_bit_1wire
*
*Rckgabe:				ausgelesenes bit
*
*Parameter:				auszulesender Multiplexer: 'a'= Multplexer A	'b'= Multiplexer B
*
*Aufgabe:				liest 1 bit vom Sensor (Kanal mu vorher mit temp_channel() gewhlt werden!)
*
*
* getestet: ja, funktioniert
*
***************************************************************************************************************************/

uint8_t read_bit_1wire(uchar multiplexer)
{
	uint8_t bit=0, sreg_temp=0;
	
	sreg_temp = SREG;		// SREG sichern
	cli();					// Interrupt aus
	
	switch(multiplexer)
	{
		case'a':										// Multiplexer A
		{
			//erzeugen einer fallenden Flanke
			
			W1_DDR |= (1<<W1_PIN_A);					// Datenrichtung auf Ausgang => 1-Wire "low"
			_delay_us(2);								// 1usec waten		minimum 1usec
			W1_DDR &= ~(1<<W1_PIN_A);					// Datenrichtung auf Eingang	=> 1Wire "high"
			_delay_us(11);								// 11usec warten 
			
			if( (W1_IN & (1<<W1_PIN_A)) )					// bit auslesen innerhalb 15usec!
				bit = 1;
			else
				bit = 0;
		}												
		break;

		case 'b':										//Multiplexer B
		{
			//erzeugen einer fallenden Flanke
			
			W1_DDR |= (1<<W1_PIN_B);					// Datenrichtung auf Ausgang => 1-Wire "low"
			_delay_us(2);								// 2usec waten		minimum 1usec
			W1_DDR &= ~(1<<W1_PIN_B);					// Datenrichtung auf Eingang	=> 1Wire "high"
			_delay_us(11);								// 11usec warten 
					
			if( (W1_IN & (1<<W1_PIN_B)) )					// bit auslesen innerhalb 15usec!
				bit = 1;
			else
				bit = 0;
		}												
		break;
	}
	
	_delay_us(40);								// 52usec warten 	min. Gesamtdauer Lesezyklus= 60usec
	_delay_us(14);								// 65-(2+11)= 52usec + 2usec recovery time => 54usec
	
	SREG = sreg_temp;							// SREG zurckschreiben
	
	return bit;									// bit zurckgeben
}






/**************************************************************************************************************************
*
*Funktionsname:			read_byte_1wire
*
*Rckgabe:				ausgelesenes Byte
*
*Parameter:				auszulesender Multiplexer (Parameter 'a' oder 'b')
*
*Aufgabe:				liest 1 Byte vom Sensor
*
*
* getestet: ja, byte immer nur 255??
*
***************************************************************************************************************************/

uint8_t read_byte_1wire(uchar multiplexer)
{
	uint8_t byte=0, i;
	
	
	
	switch(multiplexer)
	{
	
		case'a':										// Multiplexer A
		{
			for(i=0;i<=7;i++)							// 8 bit lesen
				byte |= (read_bit_1wire('a') << i);		// schiebt gelesenes bit an die Stelle i (0bis7) 
		}												// und schreibt es in die Variable byte
		break;
		
	
		
		case'b':										// Multiplexer B
		{
			for(i=0;i<=7;i++)							// 8 bit lesen
				byte |= (read_bit_1wire('b') << i);			// schiebt gelesenes bit an die Stelle i (0bis7) 
		}												// und schreibt es in die Variable byte
		break;
	
		
	}
	
	return byte;										// Rckgabe des ausgelesenen Bytes
}

	



/**************************************************************************************************************************
*
*Funktionsname:			convert_temp_1wire
*
*Rckgabe:				--
*
*Parameter:				--
*
*Aufgabe:				Sendet den Befehl "konvertiere Temparatur" an ALLE Sensoren
*						Anschl. mu min 750 msec gewartet werden!
*
*
***************************************************************************************************************************/

void convert_temp_1wire(void)
{
	uint8_t i;
	
	for (i=0;i<=7;i++)						// schalte alle Kanle der Reihe nach durch
	{
		temp_channel(i);					// schalte auf Kanal i
		write_byte_1wire(SKIP_ROM);			// schreibe den Befehl "berspringe ROM"
		write_byte_1wire(CONVERT_TEMP);		// schreibe den Befehl "konvertiere Temparatur"
	}
}


/**************************************************************************************************************************
*
*Funktionsname: 		read_scratchpad_1wire
*
*Rckgabe:				--
*
*Parameter:				--
*
*Aufgabe:				Sendet den Befehl Read Scratchpad an alle Sensoren
*
*
***************************************************************************************************************************/

void read_scratchpad_1wire(void)
{
	uint8_t i;
	
	for (i=0;i<=7;i++)						// schalte alle Kanle der Reihe nach durch
	{
		temp_channel(i);					// schalte auf Kanal i
		write_byte_1wire(SKIP_ROM);			// schreibe den Befehl "berspringe ROM"
		write_byte_1wire(READ_SCRATCHPAD);	// schreibe den Befehl "lese scratchpad"
	}
}







/**************************************************************************************************************************
*
*Funktionsname:				read_temp_1wire
*
*Rckgabe:					Temparatur
*
*Parameter:					auszulesender Kanal + Multiplexer (Bsp.: read_temp_1wire(0,'a');)
*
*Aufgabe:					Auslesen der Temparatur (Byte 0 + Byte 1) + CRC berprfung
*							Vor dem Aufrufen dieser Fkt. mu einmal ein 1wire-Reset durchgefhrt werden!
*							Mittels reset_all_1wire() oder reset_1wire()
*							Anschl. knnen die gewnschten Sensoren (Temp.) ausgelesen werden.
*							Die Temparatur wird in folgendem Format ausgegeben: 223 entspricht 22,3C
*																				-55 entspricht -5,5C
*
* getestet: ja, funktioniert
***************************************************************************************************************************/

int16_t read_temp_1wire(uint8_t channel, uchar multiplexer)
{
	uint8_t scratchpad[9], i, b, c, crc=0; 					// 9x1 Byte reservieren
	int16_t temp=0, temp1=0, zahl=0, nachkomma=0;
	char string[20];

	for(i=0;i<=8;i++)										// Indizes laufen von 0 bis 8!
		scratchpad[i]=0;									// alle Bytes auf null setzen => Feld initialisieren
		
	temp_channel(channel);									// auf den gewnschten Kanal schalten
			
	// auslesen der 9 Byte
	for(b=0;b<=8;b++)										// Indizes von Feld => 0 bis 8
		scratchpad[b] = (read_byte_1wire(multiplexer));		// liest die 9 Byte von Multiplexer A
	
	for(i=0;i<=8;i++)							
	{
		itoa(scratchpad[i] , string , 10);					
		uart_puts(string);
		uart_putc('\n');
	}
	
		
	//CRC bilden
	for(c=0;c<=8;c++)									// alle 9 Byte vom Srcratchpad werden an die CRC-Fkt. bergeben
		crc = _crc_ibutton_update(crc, scratchpad[c]);	// empfangene Daten mittels CRC berprfen CRC=0 => alles ok
														
	// CRC vergleichen
	if(crc == 0)										// wenn CRC=0 alles ok. und Temparatur ausgeben
	{
		temp = ( (scratchpad[1] << 8) | scratchpad[0]);	// Temparatur zusammensetzen (MSB+LSB)
		

		uart_putc('\n');
		itoa(temp , string , 10);				
		uart_puts(string);
		uart_putc('\n');
				
		
		
		zahl = ( (temp / 16) * 10 );			// Vorkomma x10
		
		nachkomma = temp & 0x000F;				// Nachkomma holen

		if (temp & 0x8000)						// ist Temp negativ?
		{	
			nachkomma = -nachkomma;				// wenn ja => 2er Komplement bei Nachkomma
			nachkomma = nachkomma & 0x00F;		// berflssige 1sen lschen
			nachkomma = -nachkomma;				// 2er Komplement
		}
							
		nachkomma = ((nachkomma * 10) /16);		// Nachkomma auf 1 Stelle 
		
		temp1 = (zahl + nachkomma);				// Temparatur zusammensetzen, Auflsung 1/10C

					
		itoa(temp1 , string , 10);					
		uart_puts(string);
		uart_puts("\n \n \n");
		
		return	temp1;									// return Temparatur
	}													
			
	else												// wenn CRC ungleich 0 => Lesefehler
		return (CRC_ERROR);								// return Fehlercode
}
		
		
		





/**************************************************************************************************************************
*
*Funktionsname:		show_errorcode_1wire
*
*Rckgabe:			--
*
*Parameter:			--
*
*Aufgabe:			liest den Errorcode, welcher von der Fkt. reset_all_1wire() erzeugt wird aus und zeigt diesen
*					ber die serielle Schnittstell an
*
*					0 ... alles ok
*					1 ... Multiplexer A   Fehler oder kein Sensor
*					2 ... Multiplexer B   Fehler oder kein Sensor
*					3 ... Multiplexer A+B Fehler oder kein Sensor
*
*
*getestet:			ja, funktioniert mit HTerm: newline at LF
*
***************************************************************************************************************************/

void show_errorcode_1wire(void)
{
	char string[2];							//Puffer fr itoa fr 1 Zeichen
	uint8_t i;
	
	uart_puts("\n\n\n\n**************** 1 Wire Fehlercodes *******************\n");
	uart_puts        ("*******************************************************\n\n\n\n");
	
	for(i=0;i<=7;i++)							// Alle Fehlercodes einzeln abfragen
	{
		uart_puts("Kanal ");
		itoa(i , string , 10);					// konvertiere Kanalnummer in ASCII-Code
		uart_puts(string);						// gib Kanalnummer aus
		
				
		switch(errorcode[i])					// werte Errorcode aus
		{
			case 0:								// Code 0
				uart_puts(": alles o.k.\n\n");
			break;	
			
			
			case 1:								// Code 1
				uart_puts(": Multiplexer A Fehler oder kein Sensor\n\n");
			break;
				
			
			case 2:								// Code 2
				uart_puts(": Multiplexer B Fehler oder kein Sensor\n\n");
			break;
				
			
			case 3:								// Code 3
				uart_puts(": Multiplexer A+B Fehler oder kein Sensor\n\n");
			break;
			
			
			default:
				uart_puts(": unbekannter Code\n\n");
		}	

	}
}





/**************************************************************************************************************************
*
*Funktionsname:			display_temp_1wire
*
*Rckgabe:				--
*
*Parameter:				Temparatur + Koordinaten am Display (x,y)
*
*Aufgabe:				Formatiert den Temp.-Wert, welcher von read_temp_1wire ausgelesen wurede und zeigt diesen
*						am Display an. (z.B: -10,5C)
*						Maximale Anzahl an angezeigter Zeichen zB.: -100,3C = 8 Zeichen!
*
*getestet:				ja, funktioniert
*
***************************************************************************************************************************/

void display_temp_1wire(int16_t temp , uchar x , uchar y)
{
	int16_t zahl=0;
	int8_t nachkomma=0;
	char string[7];						// Puffer fr 6 Zeichen
	//char string2[4];

	if(temp == CRC_ERROR)
	{	
		lcd_gotoxy(x,y);
		lcd_puts("CRC_ERR");
	}
	
	else
	{
		zahl = temp/10;
		itoa (zahl , string , 10);				// zahl in ASCII konvertieren
		lcd_gotoxy(x,y);						// Cursor an die gewnschte Stelle rcken
		lcd_puts(string);						// gibt die Zahl mit Vorzeichen, wenn vorhanden am Display aus
		lcd_putc('.');							// Zeichen ',' am Display ausgeben
		nachkomma = temp % 10;					// Nachkommastelle "holen"
		if(nachkomma <0)
			nachkomma = -nachkomma;
		itoa (nachkomma , string, 10);			// Nachkomma konvertieren
		lcd_puts(string);						// Nachkomma anzeigen
		lcd_putc(' ');
		lcd_putc(0xDF);							// Adresse fr ''-Zeichen von Datenblatt, ansonsten wird falsches Zeichen angezeigt
		lcd_putc('C');							// Zeichen C ausgeben
	}
}














/**************************************************************************************************************************
*
*Funktionsname:
*
*Rckgabe:
*
*Parameter:
*
*Aufgabe:
*
*
***************************************************************************************************************************/

