/**************************************************************************************************
*
*							DS18B20.h
*
*
*
*
*
***************************************************************************************************/

#ifndef F_CPU
#define F_CPU 16000000UL	//Quarzfrequenz in Herz
#endif

#ifndef _DS18B20_h
#define _DS18B20_h
#endif

#include <stdint.h>
#include <avr/io.h>
#include <util/delay.h>		
#include <avr/interrupt.h>
#include <stdio.h>
#include <util/crc16.h>
#include <stdlib.h>
#include "lcd.h"
#include "uart.h"


/*bei einer Quarzfrequenz von 16Mhz ergeben sich folgende maximale Delay-Zeiten:

_delay_us() maximal 48 usec !
_delay_ms() maximal 16 msec !


wird ein Quarz mit mehr als 16Mhz verwendet, so mssen die Delayzeiten angepasst werden!
*/

#define uchar unsigned char		//Tipparbeit sparen ;-)

uint8_t errorcode[8];			// globales Feld fr Fehlercode



//*********************** DS18B20 Commands *****************************************************************

#define SKIP_ROM			0xCC
#define CONVERT_TEMP		0x44
#define READ_SCRATCHPAD		0xBE






//********************** Fehlermeldungen *********************************************************

#define CRC_ERROR			0xFFFF




/************************ Datenleitungen Multiplexer A + Multiplexer B ****************************

Es werden analoge 1in8 Multiplexer des Typs CD 4051 verwendet, da eine Kommunikation in BEIDE 
Richtungen erforderlich ist.
Die  2 Datenleitungen der beiden Multiplexer mssen am selben Port hngen!


*/

#define W1_PIN_A	PC6
#define W1_PIN_B	PC7
#define W1_IN		PINC
#define W1_PORT		PORTC
#define W1_DDR		DDRC




/********************** Adressleitungen fr Multiplexer A + Multiplexer B *************************


Die Adressleitungen beider Multiplexer sind miteinander verbunden! D.h. es werden immer beide Multiplexer simultan 
adresiert.
Die Adressleitungen knnen an verschiedenen Ports angeschlossen werden, da jede Leitung vollstndig separat definiert.
*/

#define ADR_0_PIN	PC0			//Adressleitung 0
#define ADR_0_IN	PINC
#define ADR_0		PORTC
#define ADR_0_DDR	DDRC

#define ADR_1_PIN	PC1			//Adressleitung 1
#define ADR_1_IN	PINC
#define ADR_1		PORTC
#define ADR_1_DDR	DDRC

#define ADR_2_PIN	PC2			//Adressleitung 2
#define ADR_2_IN	PINC
#define ADR_2		PORTC
#define ADR_2_DDR	DDRC




/************  5  bentigte Leitungen fr maximal 16 Stck DS18B20 ***************************************



3 Adressleitungen und 2 Datenleitungen


Die Sensoren mssen separat mit Spannung versorgt werden!! (KEINE parasitre Versorgung!)



*/









//****************************** Funktionsprototypen **************************************************************




inline void delay_us(uint8_t,uint8_t);
uint8_t reset_1wire(void);
inline void init_1wire(void);
void temp_channel(uint8_t);
void reset_all_1wire(void);
void write_bit_1wire(uint8_t);
void write_byte_1wire(uint8_t);
uint8_t read_bit_1wire(uchar);
uint8_t read_byte_1wire(uchar);
void convert_temp_1wire(void);
void read_scratchpad_1wire(void);
int16_t read_temp_1wire(uint8_t,uchar);
void show_errorcode_1wire(void);
void display_temp_1wire(int16_t,uchar,uchar );



