/********************************************************************************************
*
*					timer1.c
*
*
*
*
* Basierend auf dem Code von Peter Dannegger
*******************************************************************************************/



#include "timer1.h"





/********************************************************************************************
*							Interrupt Service Routine TIMER1_COMPA
*
*
*
*
*Aufgabe:	Der Vergleichswert von Timer1 (OCR1A) ist so gewhlt, da er alle 4msec ereicht wird
*			und auch zum Entprellen von Tasten genutzt werden kann.
*			Diese 4msec werden zustzlich durch den Software_Teiler multipliziert, soda ein 
*			Sekundentakt entsteht.
*
*********************************************************************************************/




ISR (TIMER1_COMPA_vect)
{

#if XTAL_TIMER1 % SOFT_PRESCALER							// Rest? wenn ja: OCR1A = normaler Vergleichswert
	OCR1A = XTAL_TIMER1 / SOFT_PRESCALER - 1;
#endif

	if(--soft_prescaler == 0)						// zhle software_teiler (250) runter bei 0 ist eine sekunde vergangen
	{
		soft_prescaler = (uint8_t) SOFT_PRESCALER;	// setze software_teiler wieder auf Anfangswert
		sekunde++;									// erhhe Variable fr Sekunde
		

#if XTAL % SOFT_PRESCALER							// Rest? wenn ja: OCR1A = korrigierter Vergleiswert (nur 1x pro sec)
	OCR1A = (XTAL_TIMER1 / SOFT_PRESCALER - 1) + XTAL_TIMER1 % SOFT_PRESCALER
#endif
	}
}



/***********************************************************************************************
*
* Funktionsname:		init_timer1
*
* Rckgabe:				--
*
*Parameter:				--
*
*Aufgabe:				Initialisiert Timer1
*
***********************************************************************************************/

void init_timer1(void)
{
	TCCR1B |= 1<<WGM12 | 1<<CS10;				// CTC-Mode, Prescaler=1
	OCR1A = XTAL_TIMER1 / SOFT_PRESCALER - 1;			// Vergleichswert setzen
	TCNT1 = 0;									// Zhlregister initialisieren
	sekunde = 0;								// Sekundentakt initialisieren
	soft_prescaler = (uint8_t) SOFT_PRESCALER;	// Software Teiler setzen
	TIMSK |= 1<<OCIE1A;							// Interrupt fr Comparewert A ein
}
